"""Utility functions for the {{plugin_name}} plugin."""

from {{package_name}}.utils.formatting import format_greeting
from {{package_name}}.utils.analysis import analyze_project

__all__ = ["format_greeting", "analyze_project"]